/*
 *  bf-crypt.cpp
 *
 *  BLOWFISH CRYPTOGRAPHY
 *
 *  Copyright (c) 2006 by Dominik Erdmann - www.der-domi.de
 *
 *  Description:
 *  bf-crypt.exe encrypt/decrpyt files by using the blowfish algorithm.
 *  The encrypted file will be base64-converted.
 *
 *  Used projects:
 *  Your password will be hashed with RipeMD-160 
 *  from http://homes.esat.kuleuven.be/~bosselae/ripemd160.html.
 *  So the keylenght is always 160 bits.
 *  For blowfish cryptography I used the reference source code from SSLeay 0.6.6
 *  For base64 I used the source code from http://base64.sourceforge.net
 *
 *  encrypt - syntax:
 *  bf-crypt -e <file-to-encrypt> <encrypted-file>
 *
 *  decrypt - syntax:
 *  bf-crypt -d <file-to-decrypt> <decrypted-file>
 *
 */


// mode
#define ENCRYPT 1
#define DECRYPT 2

// ASCII
#define NUL       0x00
#define LINEFEED  0x0D
#define ABORT     0x03
#define BEEP      0x07
#define BACKSPACE 0x08

// password
#define MIN_PWD_LEN 4
#define MAX_PWD_LEN 58

#define RMDsize 160

// temporary filename
#define TMP_FILENAME "bf-crypt.tmp"

typedef unsigned char byte;
typedef unsigned long dword;


// context structure for blowfish
#ifndef __BLOWFISH_CTX
#define __BLOWFISH_CTX
typedef struct {
  unsigned long P[16 + 2];
  unsigned long S[4][256];
} BLOWFISH_CTX;
#endif


// prototypen
int getNext64BitModulo ( int lenght );
int getPwdEntry ( char *pwd );
int getPwd( char *pwd, int mode );
int getFileSize( FILE *file );
unsigned char *fillBufferFromFile( char *filename, int *lenBuffer );
void fillFileFromBuffer( unsigned char *buffer, int lenBuffer, char *filename );
unsigned char *encodeBuffer( unsigned char *buffer, int lenOld, int *lenNew );
unsigned char *decodeBuffer( unsigned char *buffer, int lenOld, int *lenNew );
void crypt( BLOWFISH_CTX *ctx, unsigned char *buffer, int len, int mode );
void clearBuffer( unsigned char *buffer, int len );
void printInfo(void);
unsigned char *RMD(byte *message);
void ctx_init( BLOWFISH_CTX *ctx, int mode );
